import type {
  UserListResponseModel,
  ListViewConfig,
  AddViewConfig,
  UserDetailResponseModel,
  LabeledValue,
  FormValues,
} from '@/helpers/models';
import {fetchAuthenticated} from '@/helpers/retrieveData';
import {useLoginStore} from '@/stores/login';

const fetchListViewConfigMocked = (): ListViewConfig => {
  const _config: ListViewConfig = {
    url: 'str',
    allowedGlobalActions: ['add'],
    searchFormOptions: {
      schools: [
        {
          value: 'DEMOSCHOOL',
          label: 'DEMOSCHOOL',
        }, {
          value: 'School1',
          label: 'Schule 1',
        },
      ],
      activationStates: [
        {
          value: 'aktiviert',
          label: 'Aktiviert',
        }, {
          value: 'deactivated',
          label: 'Deaktiviert',
        },
      ],
      roles: [
        {
          value: 'rolea',
          label: 'Role A',
        }, {
          value: 'roleb',
          label: 'Role B',
        },
      ],
    },
    columns: [
      {
        label: 'Last name',
        attribute: 'lastname',
        // sorting: 'asc',
      },
      {
        label: 'First name',
        attribute: 'firstname',
        // sorting: 'asc',
      },
      {
        label: 'User name',
        attribute: 'username',
        // sorting: 'asc',
      },
      {
        label: 'Schools',
        attribute: 'schools',
        // sorting: 'asc',
        formatter: 'list',
      },
      {
        label: 'Primary mail address',
        attribute: 'email',
        // sorting: 'asc',
      },
      {
        label: 'Activation state',
        attribute: 'disabled',
        // sorting: 'asc',
        formatter: 'activationState',
      },
      {
        label: 'Role',
        attribute: 'ucsschool_roles',
        // sorting: 'asc',
        formatter: 'list',
      },
      {
        label: 'Source UID',
        attribute: 'source_uid',
        // sorting: 'asc',
      },
      {
        label: 'Record UID',
        attribute: 'record_uid',
        // sorting: 'asc',
      },
      {
        label: 'Creation date',
        attribute: 'create_timestamp',
        // sorting: 'asc',
        formatter: 'date:true',
      },
      {
        label: 'Date of last change',
        attribute: 'modify_timestamp',
        // sorting: 'asc',
        formatter: 'date:true',
      },
      {
        label: 'Account activation date',
        attribute: 'account_activation_date',
        // sorting: 'asc',
        formatter: 'date',
      },
      {
        label: 'Account expiry date',
        attribute: 'expiration_date',
        // sorting: 'asc',
      },
      {
        label: 'Scheduled deletion date',
        attribute: 'ucsschool_purge_timestamp',
        // sorting: 'asc',
      },
    ],
  };
  console.debug('mock config', _config);

  return _config;
};

export const fetchListViewConfig = async(): Promise<ListViewConfig> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND === 'false') {
    console.debug('fetchListViewConfig. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(fetchListViewConfigMocked());
      }, 1000);
    });
  }

  const loginStore = useLoginStore();
  const answer = await fetchAuthenticated(`/ucsschool/bff-users/v1/pages/listView/settings/${loginStore.username}`);

  return await answer.json();
};



const fetchAddViewConfigMocked = (): AddViewConfig => {
  const config: AddViewConfig = {
    pages: [
      {
        label: 'General',
        name: 'general',
        fieldsets: [
          {
            label: 'Basic settings',
            name: 'general__basicSettings',
            rows: [
              [
                {
                  type: 'UInputText',
                  props: {
                    name: 'firstname',
                    label: 'First name',
                    required: true,
                    access: 'write',
                  },
                },
                {
                  type: 'UInputText',
                  props: {
                    name: 'lastname',
                    label: 'Last name',
                    required: true,
                    access: 'write',
                  },
                },
              ],
              [
                {
                  type: 'UInputText',
                  props: {
                    name: 'name',
                    label: 'User name',
                    required: true,
                    access: 'write',
                  },
                },
                {
                  type: 'UInputDate',
                  props: {
                    name: 'birthday',
                    label: 'Birthday',
                    required: false,
                    access: 'write',
                  },
                },
              ],
              [
                {
                  type: 'UInputText',
                  props: {
                    name: 'email',
                    label: 'Primary mail',
                    required: false,
                    access: 'write',
                  },
                },
              ],
              [
                {
                  type: 'UInputPassword',
                  props: {
                    name: 'password',
                    label: 'Password',
                    required: false,
                    access: 'write',
                  },
                },
              ],
              [
                {
                  type: 'UInputCheckbox',
                  props: {
                    label: 'Disabled',
                    name: 'disabled',
                    required: false,
                    access: 'write',
                  },
                },
              ],
            ],
          },
          {
            label: 'Account',
            name: 'general__account',
            rows: [
              [
                {
                  type: 'UInputDate',
                  props: {
                    name: 'accountActivationDate',
                    label: 'Activation date',
                    required: false,
                    access: 'write',
                  },
                },
              ],
              [
                {
                  type: 'UInputDate',
                  props: {
                    name: 'expirationDate',
                    label: 'Expiration date',
                    required: false,
                    access: 'write',
                  },
                },
              ],
            ],
          },
          {
            label: 'UCS@school',
            name: 'ucs@school',
            rows: [
              [
                {
                  type: 'USelect',
                  props: {
                    name: 'school',
                    label: 'Primary school',
                    required: true,
                    access: 'write',
                    options: [
                      {
                        label: 'School1',
                        value: 'school1',
                      },
                      {
                        label: 'School2',
                        value: 'school2',
                      },
                    ],
                  },
                },
              ],
              [
                {
                  type: 'UMultiSelect',
                  props: {
                    name: 'schoolClasses',
                    label: 'School classes',
                    required: true,
                    access: 'write',
                    options: [],
                  },
                },
              ],
              [
                {
                  type: 'USelect',
                  props: {
                    name: 'role',
                    label: 'Role',
                    required: true,
                    access: 'write',
                    options: [
                      {
                        label: 'Role1',
                        value: 'role1',
                      },
                      {
                        label: 'role2',
                        value: 'role2',
                      },
                    ],
                  },
                },
              ],
            ],
          },
        ],
      },
    ],
  };
  console.debug('mock config', config);
  return config;
};
export const fetchAddViewConfig = async(): Promise<AddViewConfig | null> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND === 'false') {
    console.debug('fetchAddViewConfig. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(fetchAddViewConfigMocked());
      }, 1000);
    });
  }
  const loginStore = useLoginStore();
  const answer = await fetchAuthenticated(`/ucsschool/bff-users/v1/pages/addView/settings/${loginStore.username}`);

  return await answer.json();
};

export const createUser = async(values: FormValues): Promise<boolean> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND === 'false') {
    console.debug('createUser. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(true);
      }, 1000);
    });
  }

  const answer = await fetchAuthenticated('/ucsschool/bff-users/v1/users/', {
    method: 'POST',
    headers: {
      'Content-Type': 'application/json',
    },
    body: JSON.stringify(values),
  });

  return answer.ok;
};


const fetchUsersMocked = (): UserListResponseModel[] => {
  /* eslint-disable camelcase */
  const user: UserListResponseModel = {
    id: 'user1',
    allowedActions: ['edit', 'delete', 'set_password', 'toggle_wifi', 'toggle_active'],
    attributes: {
      lastname: {
        value: 'Mustermann',
        access: 'read',
      },
      firstname: {
        value: 'Max',
        access: 'read',
      },
      username: {
        value: 'user1',
        access: 'read',
      },
      name: {
        value: 'max.mustermann',
        access: 'read',
      },
      schools: {
        value: ['DEMOSCHOOL'],
        access: 'read',
      },
      school: {
        value: 'string string string string ',
        access: 'read',
      },
      email: {
        value: 'string',
        access: 'read',
      },
      disabled: {
        value: false,
        access: 'read',
      },
      ucsschool_roles: {
        value: ['teacher:school:DEMOSCHOOL'],
        access: 'read',
      },
      source_uid: {
        value: 'KELVIN',
        access: 'none',
      },
      record_uid: {
        value: 'mmrecord',
        access: 'none',
      },
      create_timestamp: {
        value: '2022-08-26T09:46:35.422Z',
        access: 'none',
      },
      modify_timestamp: {
        value: '2022-08-26T09:46:35.422Z',
        access: 'none',
      },
      account_activation_date: {
        value: '2022-08-26T09:46:35.422Z',
        access: 'none',
      },
      expiration_date: {
        value: '2022-08-26',
        access: 'none',
      },
      ucsschool_purge_timestamp: {
        value: '2022-08-26',
        access: 'none',
      },
    },
  };
  /* eslint-enable camelcase */


  const _users: UserListResponseModel[] = [user];

  for (let x = 2; x < 500; x++) {
    const y = JSON.parse(JSON.stringify(user));
    y.id = `user${x}`;
    y.attributes.username.value = y.id;
    if (x % 2 === 0) {
      y.allowedActions = ['edit'];
    }
    _users.push(y);
  }
  return _users;
};

export const fetchUsers = async(
  school: string,
  group: string,
  role: string,
  sourceUid: string,
  state: string,
  quickSearch: string,
): Promise<UserListResponseModel[]> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND === 'false') {
    console.debug('fetchUsers. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(fetchUsersMocked());
      }, 1000);
    });
  }

  const urlSearchParams = new URLSearchParams({});
  if (group !== '') {
    urlSearchParams.append('group', group);
  }
  if (role !== '') {
    urlSearchParams.append('role', role);
  }
  if (sourceUid !== '') {
    urlSearchParams.append('sourceUid', sourceUid);
  }
  if (quickSearch !== '') {
    urlSearchParams.append('quickSearch', quickSearch);
  }
  if (state !== '') {
    urlSearchParams.append('disabled', (state === 'disabled').toString());
  }
  const answer = await fetchAuthenticated(`/ucsschool/bff-users/v1/users/search/${school}?${urlSearchParams.toString()}`);

  return await answer.json();
};

const fetchGroupsMocked = (): LabeledValue<string>[] => {
  const _groups = [
    {
      label: '1a (school class)',
      value: '1a',
    },
    {
      label: '2a (school class)',
      value: '2a',
    },
  ];
  console.log('mock groups: ', _groups);
  return _groups;
};

export const fetchGroups = async(school: string): Promise<LabeledValue<string>[] | null> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND === 'false') {
    console.debug('fetchGroups. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(fetchGroupsMocked());
      }, 1000);
    });
  }
  const answer = await fetchAuthenticated(`/ucsschool/bff-users/v1/groups/${school}`);
  if (!answer.ok) {
    // FIXME differentiate between different error causes
    return null;
  }
  const _groups = await answer.json();
  return Object.keys(_groups).map(key => ({
    label: key,
    value: _groups[key] as string,
  }));
};

const fetchUserMocked = (): UserDetailResponseModel => {
  const _user: UserDetailResponseModel = {
    url: '',
    allowedActions: ['save'],
    pages: [
      {
        label: 'General',
        name: 'general',
        fieldsets: [
          {
            label: 'Basic settings',
            name: 'basic',
            rows: [
              [
                {
                  type: 'UInputText',
                  props: {
                    name: 'firstname',
                    label: 'First name',
                    required: false,
                    access: 'write',
                  },
                },
                {
                  type: 'UInputText',
                  props: {
                    name: 'lastname',
                    label: 'Last name',
                    required: true,
                    access: 'write',
                  },
                },
              ],
              [
                {
                  type: 'UInputText',
                  props: {
                    name: 'divisNameAffix',
                    label: 'Name affix',
                    required: false,
                  },
                },
                {
                  type: 'UInputText',
                  props: {
                    name: 'name',
                    label: 'User name',
                    required: true,
                    access: 'write',
                  },
                },
              ],
              {
                type: 'UInputPassword',
                props: {
                  name: 'password',
                  label: 'Password',
                  required: false,
                  access: 'write',
                },
              },
              {
                type: 'UInputText',
                props: {
                  name: 'primaryMail',
                  label: 'Primary mail',
                  required: false,
                  access: 'write',
                },
              },
              {
                type: 'UInputCheckbox',
                props: {
                  label: 'Disabled',
                  name: 'disabled',
                  required: false,
                  access: 'write',
                },
              },
            ],
          },
          {
            label: 'Persönliche Informationen',
            name: 'personalInformation',
            rows: [
              {
                type: 'UInputText',
                props: {
                  name: 'nickname',
                  label: 'Nickname',
                  required: false,
                  access: 'write',
                },
              },
              {
                type: 'UInputText',
                props: {
                  name: 'displayName',
                  label: 'Display name',
                  required: false,
                  access: 'write',
                },
              },
              {
                type: 'UInputDate',
                props: {
                  name: 'birthday',
                  label: 'Birthday',
                  required: false,
                  access: 'write',
                },
              },
              {
                type: 'UInputText',
                props: {
                  name: 'e-mail',
                  label: 'E-mails',
                  required: false,
                },
              },
            ],
          },
          {
            label: 'UCS@school',
            name: 'ucs@school',
            rows: [
              {
                type: 'USelect',
                props: {
                  name: 'school',
                  label: 'Primary school',
                  options: [],
                  required: true,
                },
              },
              {
                type: 'UMultiSelect',
                props: {
                  name: 'schools',
                  label: 'Schools',
                  options: [],
                  required: true,
                },
              },
              {
                type: 'UMultiSelect',
                props: {
                  name: 'workgroups',
                  label: 'Work groups',
                  options: [],
                  required: false,
                },
              },
              {
                type: 'UMultiSelect',
                props: {
                  name: 'roles',
                  label: 'Roles',
                  options: [],
                  required: true,
                },
              },
              [
                {
                  type: 'UInputText',
                  props: {
                    name: 'sourceUid',
                    label: 'Source UID',
                    required: false,
                  },
                },
                {
                  type: 'UInputText',
                  props: {
                    name: 'recordUid',
                    label: 'Record UID',
                    required: false,
                  },
                },
              ],
              {
                type: 'UInputDate',
                props: {
                  name: 'ucsschoolPurgeTimestamp',
                  label: 'Scheduled deletion date',
                  required: false,
                },
              },
            ],
          },
        ],
      },
      {
        label: 'Konto',
        name: 'account',
        fieldsets: [
          {
            label: 'Allgemein',
            name: 'general',
            rows: [
              {
                type: 'UInputDate',
                props: {
                  name: 'creationDate',
                  label: 'Creation date',
                  required: false,
                },
              },
              {
                type: 'UInputDate',
                props: {
                  name: 'lastChange',
                  label: 'Date of last change',
                  required: false,
                },
              },
            ],
          },
          {
            label: 'Deaktivierung',
            name: 'deactivation',
            rows: [
              {
                type: 'UInputDate',
                props: {
                  name: 'expirationDate',
                  label: 'Expiration date',
                  required: false,
                },
              },
              {
                type: 'UInputCheckbox',
                props: {
                  name: 'disabled',
                  label: 'Disabled',
                  required: false,
                },
              },
            ],
          },
          {
            label: 'Gesperrte Anmeldung',
            name: 'gesperrteAnmeldung',
            rows: [
              {
                type: 'UInputCheckbox',
                props: {
                  name: 'pwdChangeNextLogin',
                  label: 'User has to change password on next login',
                  required: false,
                },
              },
            ],
          },
          {
            label: 'Aktivierung',
            name: 'activation',
            rows: [
              {
                type: 'UInputDate',
                props: {
                  name: 'accountActivationDate',
                  label: 'Activation date',
                  required: false,
                },
              },
            ],
          },
          {
            label: 'Passwort-Wiederherstellung',
            name: 'passwortWiederherstellungFieldset',
            rows: [
              {
                type: 'UInputText',
                props: {
                  name: 'PasswordRecoveryEmail',
                  label: 'Password recovery e-mail',
                  required: false,
                },
              },
              {
                type: 'UInputCheckbox',
                props: {
                  name: 'PasswordRecoveryEmailVerified',
                  label: 'Is the password recovery e-mail verified?',
                  required: false,
                },
              },
            ],
          },
          {
            label: 'WLAN',
            name: 'wlan',
            rows: [
              {
                type: 'UInputCheckbox',
                props: {
                  name: 'networkAccess',
                  label: 'WLAN (Radius) usage',
                  required: false,
                },
              },
            ],
          },
          {
            label: 'SAML Einstellungen',
            name: 'SAMLEinstellungen',
            rows: [
              {
                type: 'USelect',
                props: {
                  name: 'serviceprovider',
                  label: 'SAML service provider',
                  options: [],
                  required: false,
                },
              },
            ],
          },
        ],
      },
    ],
    values: {
      firstname: 'firstname',
      lastname: 'lastname',
      nickname: 'nickname',
      name: 'username',
      displayName: 'display name',
      birthday: '2020-05-03',
      password: '',
      primaryMail: null,
    },
  };
  console.debug('mock user', _user);
  return _user;
};

export const fetchUser = async(username: string): Promise<UserDetailResponseModel | null> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND === 'false') {
    console.debug('fetchUser. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(fetchUserMocked());
      }, 1000);
    });
  }

  const answer = await fetchAuthenticated(`/ucsschool/bff-users/v1/users/detail/${username}`);
  if (!answer.ok) {
    // FIXME differentiate between 404 (user does not exist) and other (backend failed in some way)
    // ... and other error cases (e.g. login token expired?)
    return null;
  }
  return await answer.json();
};

export const updateUser = async(url: string, values: FormValues): Promise<string> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND === 'false') {
    console.debug('updateUser. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    // TODO have the fail case in some way too
    return new Promise(resolve => {
      setTimeout(() => {
        resolve('');
      }, 1000);
    });
  }

  const fixedProtocolUrl = url.replace(/https?:/, location.protocol);
  const answer = await fetchAuthenticated(`${fixedProtocolUrl}`, {
    method: 'PATCH',
    headers: {
      'Content-Type': 'application/json',
    },
    body: JSON.stringify(values),
  });

  if (!answer.ok) {
    if (answer.status === 404) {
      return 'dataAccess.updateUser.error.404';
    }
    if (answer.status >= 500 && answer.status < 600) {
      return 'dataAccess.updateUser.error.5xx';
    }
    if (answer.status >= 400 && answer.status < 500) {
      return 'dataAccess.updateUser.error.4xx';
    }
  }
  return '';
};

export const deleteUsers = async(usernames: string[]): Promise<string[]> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND === 'false') {
    console.debug('deleteUsers. VITE_USE_REAL_BACKEND===false in DEV mode. Doing nothing');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve([usernames[0] as string]);
      }, 1000);
    });
  }

  const fails: string[] = [];
  const promises: Promise<Response>[] = [];
  for (const username of usernames) {
    const promise = fetchAuthenticated(`/ucsschool/bff-users/v1/users/${username}`, {
      method: 'DELETE',
    });
    promise.then(res => {
      if (!res.ok) {
        fails.push(username);
      }
    });
    promises.push(promise);
  }
  await Promise.all(promises);
  return fails;
};

